@@ -3,14 +3,8 @@
 const msg_t MSGT = { .ok = TRUE };
 const msg_t MSGF = { .ok = FALSE };
 
-msg_t Msg(const char *str) {
-    msg_t msg = MSGF;
-    strncpy(msg.msg, str, CMSGLEN);
-    return msg;
-}
-
-msg_t Msgf(const char *fmt, ...) {
-    msg_t msg = MSGF;
+msg_t make_msg(const char *fmt, ...) {
+    msg_t msg = { .ok = FALSE };
     va_list vargs;
     va_start(vargs, fmt);
     vsnprintf(msg.msg, CMSGLEN, fmt, vargs);
@@ -18,11 +12,7 @@ msg_t Msgf(const char *fmt, ...) {
     return msg;
 }
 
-SEXP CheckResult(const char *str) {
-    return ScalarString(mkChar(str));
-}
-
-SEXP CheckResultf(const char *fmt, ...) {
+SEXP make_result(const char *fmt, ...) {
     char msg[CMSGLEN];
     va_list vargs;
     va_start(vargs, fmt);
@@ -31,6 +21,24 @@ SEXP CheckResultf(const char *fmt, ...) {
     return ScalarString(mkChar(msg));
 }
 
+const char * guessType(SEXP x) {
+    SEXP attr = getAttrib(x, R_ClassSymbol);
+    if (!isNull(attr))
+        return CHAR(STRING_ELT(attr, 0));
+
+    attr = getAttrib(x, R_DimSymbol);
+    if (!isNull(attr))
+        return length(attr) == 2 ? "matrix" : "array";
+
+    return type2char(TYPEOF(x));
+}
+
+SEXP make_type_error(SEXP x, const char *expected) {
+    char msg[CMSGLEN];
+    snprintf(msg, CMSGLEN, "Must be of type '%s', not '%s'", expected, guessType(x));
+    return ScalarString(mkChar(msg));
+}
+
 SEXP mwrap(msg_t msg) {
     if (msg.ok)
         return ScalarLogical(TRUE);
